// Использование сокетов на примере создания сервера возвращающего строчку в заглавных буквах.
// Версия 0.1
 
#include <stdio.h>
#include <string.h>    // strncmp
#include <arpa/inet.h>

#include <ctype.h> // toupper
 
int main(int argc , char *argv[])
{
	// Создаем сокет, т.е. сетевое подключение.
	int s_srv = socket(AF_INET, // IPv4
		SOCK_STREAM, // потоковый, т.е. TCP
		0 /* протокол по умолчанию  */);

	// Проверяем успешность создания серверного сокета.
	if( s_srv == -1 )
	{
		printf("Сокет не удалось создать.");
		return -1;
	}

	// Структура для указания адреса сервера.
	struct sockaddr_in srv;
	memset( &srv, 0, sizeof( srv ) );

	srv.sin_family = AF_INET; // Для IPv4.
	srv.sin_addr.s_addr = INADDR_ANY; // Любой сетевой интерфейс.
	srv.sin_port = htons( 1234 ); // Наш порт.

	// Привязка порта к указанному сокету.
	if( bind( s_srv, (struct sockaddr *)&srv, sizeof(srv) ) < 0 )
	{
		printf("Порт не удалось привязать к сокету.");
		return -1;
	}
	printf("Привязка выполнена.\n");

	// Настраиваем размер очереди на подключение.
	listen( s_srv, 3);

	printf("Ожидаем подключения от клиента.\n");

	// Указываем размер структуры. Значение может изменится после вызова accept.
	int c = sizeof(struct sockaddr_in);
	struct sockaddr_in clnt;
	// Ожидаем и принятие подключения от клиента.
	int s_clnt = accept(s_srv, (struct sockaddr *)&clnt, (socklen_t*)&c);
	if( s_clnt < 0 )
	{
		printf("Не удалось установить соединение с клиентом.\n");
		return 1;
	}
	printf("Соединение выполнено.\n");

	// Получаем сообщение от клиента.
	int rcv_sz;
	char clnt_msg[ 2000 ];
	while( ( rcv_sz = recv( s_clnt , clnt_msg , 2000 , 0) ) > 0 )
	{
		// Получили сообщение размера rcv_sz.
		// Если получили строчку с нужным кючевым словом STOP
		if( strncmp( clnt_msg, "STOP", rcv_sz < 4 ? rcv_sz : 4 ) == 0 )
			break;// то выходим из цикла.
		// Иначе обрабатываем строчку. В данном случае
		for( int i = 0; i < rcv_sz; i++)// каждый символ
			clnt_msg[ i ] = toupper( clnt_msg[ i ] ); // делается заглавным.
		// Передаем сообщение обратно клиенту. Размер совпадает с полученной строчкой.
		send( s_clnt , clnt_msg , rcv_sz, 0 );
	}

	if( rcv_sz == 0 )
	{
		printf("Клиент отсоединился.\n");
		fflush(stdout);
	}
	else if( rcv_sz == -1 )
	{
		printf("Ошибка в получении данных.\n");
	}

	return 0;
}


